#include "Cyborg.h"
#include "../Game/Engine.h"
#include "../Game/StdInc.h"
#include "../Graphics/Graphics.h"
#include "../Graphics/Animations.h"
#include "../Player/Player.h"
#include "../Sound/SoundManager.h"

#include <D3dx9math.h>

using namespace _GRAPHICS;
using namespace _PHYSICS;
using _SPRITE::Sprite;
using _TIME::TickTimer;

namespace _AI
{

Cyborg::Cyborg(const D3DXVECTOR3& pos) : Enemy(GAME.shallowSprite(_SPRITE::CYBORG),
										 PhysicalObject(pos), 2, 2)
{
	setState(AI::MOVE_STATE);
	setVelocity((-5.f)/30.f, 0);
	facingRight = false;
}

Cyborg::~Cyborg()
{
	SOUNDS.stop( _SOUND::CYBORG_CHAINSAW_IDLE );
	SOUNDS.stop( _SOUND::CYBORG_CHAINSAW_BUZZ_LOOP );
}

void Cyborg::execState()
{
	switch(State())
	{
		case AI::MOVE_STATE:
		{
			move();
			break;
		}
		case AI::ATTACK_STATE:
		{
			attack();
			break;
		}
		case AI::DEATH_STATE:
		{
			die();
			break;
		}
	}
}
void Cyborg::attack()
{
	setVelocity(0.f);

	if(TICK_TIMER.check(mTimer) > 1300)
	{
		setAnimation(ENEMY_ATTACKING);
		TICK_TIMER.reset(mTimer);
		if ( SOUNDS.isPlaying( _SOUND::CYBORG_CHAINSAW_IDLE ) )
			SOUNDS.stop( _SOUND::CYBORG_CHAINSAW_IDLE );
		SOUNDS.play( _SOUND::CYBORG_CHAINSAW_BUZZ_LOOP, 0 );
	}
}

void Cyborg::move()
{
	if(Pos().x>FURY.Pos().x)
	{
		setVelocity( -0.3f);
		setAnimation(ENEMY_MOVING);
	}
	else if(Pos().x<FURY.Pos().x)
	{
		setVelocity(0.3f);
		setAnimation(ENEMY_MOVING);
	}
	else
	{
		setVelocity(0);
		setAnimation(ENEMY_STANDING);
	}

	if ( SOUNDS.isPlaying( _SOUND::CYBORG_CHAINSAW_BUZZ_LOOP ) )
		SOUNDS.stop( _SOUND::CYBORG_CHAINSAW_BUZZ_LOOP );
	SOUNDS.play( _SOUND::CYBORG_CHAINSAW_IDLE, 0 );
}

void Cyborg::updateAI()
{
	D3DXVECTOR3 dist(FURY.Pos() - Pos());
	float distance = D3DXVec3LengthSq(&dist);
	if (State() == AI::ATTACK_STATE)
	{
		if (distance > CYBORG_DIST)
			setState(AI::MOVE_STATE);
	}
	else
	{
		if (distance <= CYBORG_DIST)
			setState(AI::ATTACK_STATE);
	}

	execState();
}

}
