#include "Dreadnaught.h"
#include "../Game/Engine.h"
#include "../Player/Player.h"
#include "Bullet.h"
#include <list>
using _TIME::TickTimer;

#define DREAD_MOUTH 250
#define DREAD_EYES 100

enum DREAD_ANIMATIONS {
	DREAD_IDLE,
	DREAD_GAT,
	DREAD_MISSILE,
	DREAD_LASER
};

namespace _AI
{

Dread::Dread(const PhysicalObject& po) : Boss(GAME.shallowSprite(_SPRITE::DREAD_BOSS), po, DREAD_HEALTH, 1, 1, 200), attackState(PRIM_STD)
{
	patternTimer = TICK_TIMER.getTimer();
	shotCount = 0;
	patternDone = true;
}

void Dread::execState()
{
	if(patternDone)
	{
		TICK_TIMER.reset(patternTimer);
		patternDone=false;
		shotCount = 0;
	}

	switch(attackState)
	{
		case PRIM_STD:
			primaryStd();
			break;
		case PRIM_HVY:
			primaryHvy();
			break;
		case SCND_STD:
			secondaryStd();
			break;
		case SCND_HVY:
			secondaryHvy();
			break;
		case SUPER:
			superAttack();
			break;
	}	
}

void Dread::setState(DREAD_ATTACK_STATE ds)
{
	attackState = ds;
	TICK_TIMER.reset(patternTimer);
	patternDone = false;
	shotCount = 0;
}

void Dread::updateAI(void)
{
	if(State() == AI::DEATH_STATE)
	{
		die();
		return;
	}

	DREAD_ATTACK_STATE newState = attackState;
	switch(attackState)
	{
		case PRIM_STD:
			if(Health() <= DREAD_HEALTH/2) 
				newState = SCND_HVY;
			else if(Health() <= DREAD_HEALTH/4)  
				newState = SUPER;
			else if (hitTime.size() >= DREAD_RAGE_HIT)	
			{
				newState = PRIM_HVY;
				hitTime.clear();
			}
			break;

		case PRIM_HVY:
			if(Health() <= DREAD_HEALTH/2)
				newState = SCND_HVY;
			else if(patternDone == true)
				newState = PRIM_STD;
			break;

		case SCND_STD:
			if(Health() <= DREAD_HEALTH/4)
				newState = SUPER;
			else if (hitTime.size() >= DREAD_RAGE_HIT)	
			{
				newState = SUPER;
				hitTime.clear();
			}
			break;

		case SCND_HVY:
			if(Health() <= DREAD_HEALTH/4)
				newState = SUPER;
			else if(patternDone == true)
				newState = SCND_STD;
			break;

		case SUPER:
			if(patternDone == true)
				newState = SCND_STD;
			break;
	}

	if(newState != attackState)
		setState(newState);

	execState();
}

void Dread::hitEvent()
{
	if (attackState == PRIM_STD || attackState == SCND_STD)
	{
		// Use current time for time of collision - change later when
		// physics is implemented correctly and calculates hit time
		double elapsedFightTime = TICK_TIMER.check(fightTime);

		// Add the current hit time
		hitTime.push_back(elapsedFightTime);

		// Correct the time range
		while (elapsedFightTime- hitTime.front() > DREAD_RAGE_TIME)
		{
			hitTime.pop_front();
		}
	}
}

void Dread::playerBulletCollision(_PHYSICS::CollisionResult &result)
{
	changeHealth(-1);
	hitEvent();
}

void Dread::playerAutomaticBulletCollision(_PHYSICS::CollisionResult &result)
{
	changeHealth(-1);
	hitEvent();
}

void Dread::playerSpreadBulletCollision(_PHYSICS::CollisionResult &result)
{
	changeHealth(-2);
	hitEvent();
}

void Dread::shoot(float x, float y, const D3DXVECTOR3 &dir)
{
	_AI::Bullet *b = new _AI::Bullet(GAME.shallowSprite(_SPRITE::ENEMY_BULLET), x, y, dir);
	b->setAnimation(0);
	GAME.addSprite(b);
}

void Dread::primaryStd()
{
	if(shotCount<15)
	{
		shotCount++;
		setAnimation(DREAD_GAT);
		shoot(X(), Y() + DREAD_MOUTH + (rand()%10), D3DXVECTOR3(-1,0,0));
	}
	else if(TICK_TIMER.check(patternTimer) > MilliToNano(4000))
	{
		patternDone = true;
		shotCount = 0;
	}
	else
		setAnimation(DREAD_IDLE);
}

void Dread::primaryHvy()
{
	if(shotCount<35)
	{
		setAnimation(DREAD_LASER);
		shoot(X(), Y() + DREAD_MOUTH + (rand()%100), D3DXVECTOR3(-1,0,0));
		shoot(X(), Y() + DREAD_MOUTH + (rand()%100), D3DXVECTOR3(-1,0,0));
		shotCount++;
	}
	else if(TICK_TIMER.check(patternTimer) > MilliToNano(3000))
	{
		patternDone = true;
		shotCount = 0;
	}
}

void Dread::secondaryStd()
{
	if(shotCount<20)
	{
		setAnimation(DREAD_LASER);
		shoot(X(), Y() + DREAD_EYES,  D3DXVECTOR3(FURY.Pos().x-50,FURY.Pos().y-75, 0) - Pos());
		shotCount++;
	}
	else if(TICK_TIMER.check(patternTimer) > MilliToNano(3000))
	{
		patternDone = true;
		shotCount = 0;
	}
	else
		setAnimation(DREAD_IDLE);
}

void Dread::secondaryHvy() 
{
	if(shotCount < 2)
	{
		shoot(X(), Y() + DREAD_EYES - 40, D3DXVECTOR3(FURY.Pos().x-50,FURY.Pos().y-75, 0) - Pos());
		shotCount++;
	}
	else if(TICK_TIMER.check(patternTimer) < MilliToNano(1000))
	{
		patternDone = true;
		shotCount = 0;
	}
	else
		setAnimation(DREAD_IDLE);

}
void Dread::superAttack() 
{
	if(shotCount<10)
	{
		setAnimation(DREAD_LASER);
		shoot(X(), Y() + DREAD_EYES, D3DXVECTOR3(FURY.Pos().x-50,FURY.Pos().y-75, 0) - Pos());
		shoot(X(), Y() + DREAD_MOUTH + (rand()%100), D3DXVECTOR3(-1,0,0));
		shoot(X(), Y() + DREAD_MOUTH + (rand()%100), D3DXVECTOR3(-1,0,0));
		shotCount++;
	}
	else if(TICK_TIMER.check(patternTimer) > MilliToNano(2000))
	{
		patternDone = true;
		shotCount = 0;
	}
	else
		setAnimation(DREAD_IDLE);
}

}